package com.kingdee.eas.custom.expandassess.service;

import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.metadata.entity.*;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.basedata.person.PersonInfo;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.expandassess.*;
import com.kingdee.eas.custom.expandassess.app.AssessRatingEnum;
import com.kingdee.eas.custom.expandassess.app.ExpandStatusEunm;
import com.kingdee.eas.custom.expandassess.app.PeriodStatusEnum;
import com.kingdee.eas.custom.performancenew.ExamineGradePersonEntryCollection;
import com.kingdee.eas.custom.performancenew.ExamineGradePersonEntryFactory;
import com.kingdee.eas.custom.performancenew.ExamineGradePersonEntryInfo;
import com.kingdee.eas.custom.performancenew.IExamineGradePersonEntry;
import com.kingdee.eas.custom.performancenew.app.scoreEnum;
import com.kingdee.eas.framework.CoreBaseInfo;
import com.kingdee.eas.util.app.ContextUtil;
import com.kingdee.shr.base.syssetting.app.io.fileImport.BaseColumnInfo;
import com.kingdee.shr.base.syssetting.app.io.fileImport.BaseImportService;
import com.kingdee.shr.base.syssetting.app.io.fileImport.BaseRowInfo;
import com.kingdee.shr.base.syssetting.app.io.fileImport.ImportException;
import com.kingdee.shr.ml.util.SHRServerResource;
import com.kingdee.util.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author qingwu
 * @date 2024/5/20
 * @apiNote չԱ ˵ģ
 */
public class ExpandAssessKHRImportService extends BaseImportService {
    private static final SelectorItemCollection SIC = new SelectorItemCollection();
    private static final SelectorItemCollection UPDATESIC = new SelectorItemCollection();

    static {
        SIC.add("adminorg.name");
        SIC.add("assessor.name");
        SIC.add("shareliftres");
        SIC.add("shareliftzy");
        SIC.add("shareliftlc");
        SIC.add("unitelift");
        SIC.add("oneLift");

        UPDATESIC.add("oneLift");
        UPDATESIC.add("shareliftres");
        UPDATESIC.add("shareliftzy");
        UPDATESIC.add("shareliftlc");
        UPDATESIC.add("unitelift");
        UPDATESIC.add("contracts");
    }

    @Override
    protected void verifyRow(BaseRowInfo row) throws ImportException {
        super.verifyRow(row);
        String assessor = row.getValueOfString("assessor");
        PersonInfo assessorObject = (PersonInfo) row.getValueOfObject("assessor");
        String adminOrg = row.getValueOfString("adminOrg");
        String parentId = getCustomParam("parentId");
        Context ctx = getContext();
        String currentPersonId = ContextUtil.getCurrentUserInfo(ctx).getPerson().getId().toString();
        if (!currentPersonId.equals(assessorObject.getId().toString())) {
            throw new ImportException("ʧܣ");
        }
        if (!StringUtils.isEmpty(assessor) && !StringUtils.isEmpty(adminOrg)) {
            try {
                IExpandAssessEntry iExpandAssessEntry = ExpandAssessEntryFactory.getLocalInstance(getContext());
                FilterInfo filterInfo = new FilterInfo();
                filterInfo.getFilterItems().add(new FilterItemInfo("assessor.name", assessor));
                filterInfo.getFilterItems().add(new FilterItemInfo("adminOrg.name", adminOrg));
                filterInfo.getFilterItems().add(new FilterItemInfo("parent.id", parentId));
                SelectorItemCollection sic = new SelectorItemCollection();
                sic.add("parent.periodStatus");
                sic.add("asseStatus");
                EntityViewInfo entityViewInfo = EntityViewInfo.getInstance(filterInfo, sic, null);
                ExpandAssessEntryInfo entryInfo = iExpandAssessEntry.getExpandAssessEntryCollection(entityViewInfo).get(0);
                ExpandAssessInfo parent = entryInfo.getParent();
                if (parent.getPeriodStatus() == PeriodStatusEnum.LOCK) {
                    throw new ImportException("ɵ");
                }
                if (entryInfo.getAsseStatus() == ExpandStatusEunm.ASSESUBMINT) {
                    throw new ImportException("ύɵ");
                }
            } catch (BOSException e) {
                throw new ImportException(e);
            }

        }
    }

    @Override
    protected void submitData(CoreBaseInfo coreBaseInfo) throws ImportException {
        try {
            IExpandAssessEntry iExpandAssessEntry = ExpandAssessEntryFactory.getLocalInstance(getContext());
            iExpandAssessEntry.updatePartial(coreBaseInfo, UPDATESIC);
        } catch (EASBizException var4) {
            ImportException importException = new ImportException(var4.getMessage(getContext().getLocale()), (Throwable) var4);
            throw importException;
        } catch (BOSException var5) {
            ImportException importException = new ImportException(SHRServerResource.getString("com.kingdee.shr.base.syssetting.CommonserviceResource", "save_fails_bos", getContext()), (Throwable) var5);
            throw importException;
        } catch (Exception var6) {
            ImportException importException = new ImportException(SHRServerResource.getString("com.kingdee.shr.base.syssetting.CommonserviceResource", "save_submit_fails", getContext()), var6);
            throw importException;
        }
    }

    /**
     * ģɺִ
     *
     * @param columnInfoMap
     * @param wb
     * @param sheet
     */
    @Override
    public void completeGenerateExcel(Map<String, BaseColumnInfo> columnInfoMap, XSSFWorkbook wb, XSSFSheet sheet) {
        Context ctx = getContext();
        String parentId = getCustomParam("parentId");
        if (StringUtils.isEmpty(parentId)) {
            throw new RuntimeException("չԱIDΪգ");
        }
        int beginRow = 4;
        try {
            //ǰ¼Աid
            String currentPersonId = ContextUtil.getCurrentUserInfo(ctx).getPerson().getId().toString();
            //˵ǰ״̬Ϊ˵ڵ¼˵
            FilterInfo filterInfo = new FilterInfo();
            FilterItemCollection filterItems = filterInfo.getFilterItems();
            filterItems.add(new FilterItemInfo("parent.id", parentId));
            filterItems.add(new FilterItemInfo("asseStatus", "1", CompareType.NOTEQUALS));
            filterItems.add(new FilterItemInfo("asseStatus", "4", CompareType.NOTEQUALS));
            filterItems.add(new FilterItemInfo("assessor.id", currentPersonId));
            //SorterItemCollection sorterItemCollection = new SorterItemCollection();
            //sorterItemCollection.add(new SorterItemInfo("rankings"));
            EntityViewInfo entityViewInfo = EntityViewInfo.getInstance(filterInfo, SIC, null);
            IExpandAssessEntry iExpandAssessEntry = ExpandAssessEntryFactory.getLocalInstance(getContext());
            //Ч˷¼ʵ
            ExpandAssessEntryCollection entryCollection = iExpandAssessEntry.getExpandAssessEntryCollection(entityViewInfo);
            Map<Integer, String> columnIndexMap = getColumnIndexFn(columnInfoMap);
            int fillColumnCount = columnInfoMap.size();
            for (int i = 0; i < entryCollection.size(); i++) {
                XSSFRow xSSFRow = sheet.createRow(i + beginRow);
                ExpandAssessEntryInfo expandAssessEntryInfo = entryCollection.get(i);
                for (int j = 0; j < fillColumnCount; j++) {
                    String value = null;
                    Cell createCell = xSSFRow.createCell(j);
                    String columnName = columnIndexMap.get(Integer.valueOf(j));
                    Object obj = expandAssessEntryInfo.get(columnName);
                    if (obj != null) {
                        if (obj instanceof CoreBaseInfo) {
                            CoreBaseInfo coreBaseInfo = (CoreBaseInfo) expandAssessEntryInfo.get(columnName);
                            //value = coreBaseInfo.get("number") + "##" + coreBaseInfo.get("name");
                            value = (String) coreBaseInfo.get("name");
                        } else if (obj instanceof BigDecimal) {
                            BigDecimal bigDecimal = (BigDecimal) obj;
                            value = bigDecimal.setScale(2).toPlainString();
                        } else {
                            if ("checkRate".equals(columnName)) {
                                value = AssessRatingEnum.getEnum((String) obj).getAlias();
                            } else {
                                value = obj.toString();
                            }
                        }
                    }
                    createCell.setCellValue(value);
                }
            }
        } catch (BOSException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    protected Map<Integer, String> getColumnIndexFn(Map<String, BaseColumnInfo> columnInfoMap) {
        Map<Integer, String> columnIndexMap = new HashMap<>();
        Iterator<String> keys = columnInfoMap.keySet().iterator();
        int var4 = 0;
        while (keys.hasNext()) {
            String key = keys.next();
            columnIndexMap.put(Integer.valueOf(var4++), key);
        }
        return columnIndexMap;
    }

    @Override
    protected String getPKSelectString(BaseRowInfo row) {
        String pkSelectString = super.getPKSelectString(row);
        String parentId = getCustomParam("parentId");
        if (StringUtils.isEmpty(parentId)) {
            throw new RuntimeException("չԱIDΪգ");
        }
        return pkSelectString + " and parent.id ='" + parentId + "'";
    }
}
